@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: Mailer.py 10241 2017-09-01 14:08:47Z David $
+# $Id: Mailer.py 10258 2017-09-04 08:44:52Z David $
 #
 # Copyright (c) 2014 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -13,7 +13,7 @@
 #
 # $Author: David $ (Last)
 # $Date: 2013-01-30 12:13:45 +0800$
-# $Revision: 10241 $
+# $Revision: 10258 $
 
 import os
 import random
@@ -92,7 +92,7 @@ class CloudSMTP(object):
                 mail.send(fail_silently=False)
 
                 extra = mail.extra_headers
-                messageId = extra.get('messageId', '')
+                messageId = extra.get('message_id', '')
                 requestId = extra.get('request_id', '')
                 message = extra.get('message', '')
 
