@@ -1,8 +1,8 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: views.py 10198 2017-08-19 16:19:02Z David $
+# $Id: views.py 10247 2017-09-03 17:57:32Z David $
 #
-# Copyright (c) 2012 Nuwa Information Co., Ltd, All Rights Reserved.
+# Copyright (c) 2017 Nuwa Information Co., Ltd, All Rights Reserved.
 #
 # Licensed under the Proprietary License,
 # you may not use this file except in compliance with the License.
@@ -12,10 +12,12 @@
 # limitations under the License.
 #
 # $Author: David $
-# $Date: 2017-08-20 00:19:02 +0800 (Sun, 20 Aug 2017) $
-# $Revision: 10198 $
+# $Date: 2017-09-04 01:57:32 +0800 (Mon, 04 Sep 2017) $
+# $Revision: 10247 $
 
 import re
+import time
+
 from urlparse import urljoin
 
 from django.contrib.sites.models import Site
@@ -39,6 +41,7 @@ cloudSrv = CloudService.getInstance()
 def _getDomain():
     site = Site.objects.get_current()
     domain = site.domain.replace('.', '-')
+    domain = domain.replace(':', '-')
     return domain
 
 _domain = _getDomain()
@@ -46,10 +49,13 @@ _domain = _getDomain()
 AWS_ACCESS_KEY_ID = settings.AWS_ACCESS_KEY_ID
 AWS_SECRET_ACCESS_KEY = settings.AWS_SECRET_ACCESS_KEY
 
+
 BOUNCES_TOPIC_NAME = getattr(settings, 'BOUNCES_TOPIC_NAME', 'bounces')
 BOUNCES_TOPIC_NAME = '%s_%s' % (_domain, BOUNCES_TOPIC_NAME)
 COMPLAINTS_TOPIC_NAME = getattr(settings, 'COMPLAINTS_TOPIC_NAME', 'complaints')
 COMPLAINTS_TOPIC_NAME = '%s_%s' % (_domain, COMPLAINTS_TOPIC_NAME)
+DELIVERIES_TOPIC_NAME = getattr(settings, 'DELIVERIES_TOPIC_NAME', 'deliveries')
+DELIVERIES_TOPIC_NAME = '%s_%s' % (_domain, DELIVERIES_TOPIC_NAME)
 
 SUBSCRIBE_RETRY_COUNT = getattr(settings, 'SUBSCRIBE_RETRY_COUNT', 3)
 
@@ -148,6 +154,12 @@ def doVerifyEmailAddress(email):
                 if getNameFromArn(complaintTopic) == COMPLAINTS_TOPIC_NAME:
                     checkComplaint = True
 
+            deliveryTopic = notifyAttrValues.get('DeliveryTopic', '')
+            if deliveryTopic:
+                checkDelivery = False
+                if getNameFromArn(deliveryTopic) == DELIVERIES_TOPIC_NAME:
+                    checkDelivery = True
+
             if (checkBounce is False) or (checkComplaint is False):
                 response['continue'] = True
                 response['message'] += _(
@@ -259,6 +271,11 @@ def checkVerifiedEmailAddress(request):
                 if getNameFromArn(complaintTopic) == COMPLAINTS_TOPIC_NAME:
                     checkComplaint = True
 
+            deliveryTopic = notifyAttrValues.get('DeliveryTopic', '')
+            if deliveryTopic:
+                if getNameFromArn(deliveryTopic) == DELIVERIES_TOPIC_NAME:
+                    checkDelivery = True
+
             if checkBounce and checkComplaint:
                 return response
             else:
@@ -298,6 +315,16 @@ def recordStatus(messageId, destination, status):
 
         obj = cloudSrv.getCloudMailInformation(
             messageId=messageId, errorOnNotFound=False)
+
+        bufferTime = 0
+
+        while not obj and bufferTime <= 15:
+            time.sleep(5)
+            bufferTime += 5
+
+            obj = cloudSrv.getCloudMailInformation(
+                messageId=messageId, errorOnNotFound=False)
+
         if obj:
             newsletter = obj.newsletter
             contact = obj.contact
@@ -324,33 +351,6 @@ def recordRejectedStatus(messageId, destination):
 def recordComplaintStatus(messageId, destination):
     recordStatus(messageId, destination, ContactMailingStatus.COMPLAINT)
 
-def isBounceType(bounceType, bounceSubType):
-
-    if bounceType == 'Undetermined' and bounceSubType == 'Undetermined':
-        # Bounce.
-        return True
-    elif bounceType == 'Permanent' and bounceSubType == 'General':
-        # Bounce.
-        return True
-    elif bounceType == 'Permanent' and bounceSubType == 'NoEmail':
-        # Bounce.
-        return True
-    elif bounceType == 'Transient' and bounceSubType == 'General':
-        # Bounce.
-        return True
-    elif bounceType == 'Transient' and bounceSubType == 'MailboxFull':
-        # Rejected.
-        return False
-    elif bounceType == 'Transient' and bounceSubType == 'MessageToolarge':
-        # Rejected.
-        return False
-    elif bounceType == 'Transient' and bounceSubType == 'ContentRejected':
-        # Rejected.
-        return False
-    elif bounceType == 'Transient' and bounceSubType == 'AttachmentRejected':
-        # Rejected.
-        return False
-
 import sys
 import traceback
 from urllib import urlopen
@@ -410,36 +410,38 @@ def handleSNS(request):
                 message = serializer.deserialize(message)
 
             notificationType = message.get('notificationType', '')
+
+            amazonLogger.info("Message type: %s" % notificationType)
+
+            mail = message.get('mail', {})
+            destination = mail.get('destination', [])
+            messageId = mail.get('messageId', '')
+
             if notificationType == 'Bounce': # Bounced.
-                amazonLogger.info("Message type: %s" % notificationType)
-
-                mail = message.get('mail', {})
-                destination = mail.get('destination', [])
-                messageId = mail.get('messageId', '')
-
-                # Get bounce's type.
-                bounceType = message.get('bounceType', '')
-                bounceSubType = message.get('bounceSubType', '')
-                isBounce = isBounceType(bounceType, bounceSubType)
-                if isBounce:
-                    # record!
-                    amazonLogger.info("Got bounce, need record...")
-                    recordBouncedStatus(messageId, destination)
-                else:
-                    # record!
-                    amazonLogger.info("Got rejected, need record...")
-                    recordRejectedStatus(messageId, destination)
+                # record!
+                amazonLogger.info("Got bounce, need record...")
+                recordBouncedStatus(messageId, destination)
 
             elif notificationType == 'Complaint': # Complaints.
-                amazonLogger.info("Message type: %s" % notificationType)
-
-                mail = message.get('mail', {})
-                destination = mail.get('destination', [])
-                messageId = mail.get('messageId', '')
                 # Record.
                 amazonLogger.info("Got complaint, need record...")
                 recordComplaintStatus(messageId, destination)
 
+            elif notificationType == 'Reject': # Rejects.
+                amazonLogger.info("Got reject, need record...")
+                recordRejectedStatus(messageId, destination)
+
+            elif notificationType == 'Delivery': # Deliveries.
+                amazonLogger.info("Got delivery, check if need record...")
+                delivery = message.get('delivery', '')
+                smtpResponse = delivery.get('smtpResponse', '')
+
+                errorCodeList = ['450', '550', '553']
+                if any(x in smtpResponse for x in errorCodeList):
+                    amazonLogger.info("Got delivery error, need record...")
+                    recordRejectedStatus(messageId, destination)
+
+
         elif topicType == 'UnsubscribeConfirmation':
             amazonLogger.info("Topic type: %s" % topicType)
             # Handle UnsubscribeConfirmation message.
@@ -465,6 +467,10 @@ def getBounces(request):
 def getComplains(request):
     return handleSNS(request)
 
+@csrf_exempt
+def getDeliveries(request):
+    return handleSNS(request)
+
 def checkNotificationTopic(email, useCache=False):
     """
     Check notification topic.
@@ -485,7 +491,11 @@ def checkNotificationTopic(email, useCache=False):
         if getNameFromArn(complaintTopic) != COMPLAINTS_TOPIC_NAME:
             complaintTopic = ''
 
-        if bounceTopic and complaintTopic:
+        deliveryTopic = notifyAttrValues.get('DeliveryTopic', '')
+        if getNameFromArn(deliveryTopic) != DELIVERIES_TOPIC_NAME:
+            deliveryTopic = ''
+
+        if bounceTopic and complaintTopic and deliveryTopic:
             if useCache:
                 # Set cache.
                 setSenderByCache(email)
@@ -519,19 +529,22 @@ def doSetNotificationTopic(email, useCache=False):
     try:
         bounceTopic = ''
         complaintTopic = ''
+        deliveryTopic = ''
 
         ses = SES(AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY)
         # Check topics.
         sns = SNS(AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY)
 
         for topic in sns.getAllTopicArns():
-            if bounceTopic and complaintTopic:
+            if bounceTopic and complaintTopic and deliveryTopic:
                 break
             topicName = getNameFromArn(topic)
             if topicName == BOUNCES_TOPIC_NAME:
                 bounceTopic = topic
             if topicName == COMPLAINTS_TOPIC_NAME:
                 complaintTopic = topic
+            if topicName == DELIVERIES_TOPIC_NAME:
+                deliveryTopic = topic
 
         if not bounceTopic: # Create a topic of bounces.
             bounceTopic = sns.createTopic(BOUNCES_TOPIC_NAME)
@@ -539,9 +552,13 @@ def doSetNotificationTopic(email, useCache=False):
         if not complaintTopic: # Create a topic of complaints.
             complaintTopic = sns.createTopic(COMPLAINTS_TOPIC_NAME)
 
+        if not deliveryTopic: # Create a topic of complaints.
+            deliveryTopic = sns.createTopic(DELIVERIES_TOPIC_NAME)
+
         # Check subscription.
         hasBounceTopicExist = False
         hasComplaintTopicExist = False
+        hasDeliveryTopicExist = False
         for n in range(SUBSCRIBE_RETRY_COUNT):
             if hasBounceTopicExist and hasComplaintTopicExist:
                 break
@@ -551,6 +568,9 @@ def doSetNotificationTopic(email, useCache=False):
             if not hasComplaintTopicExist:
                 hasComplaintTopicExist = sns.checkSubscriptionExist(
                     complaintTopic, COMPLAINTS_TOPIC_NAME)
+            if not hasDeliveryTopicExist:
+                hasDeliveryTopicExist = sns.checkSubscriptionExist(
+                    deliveryTopic, DELIVERIES_TOPIC_NAME)
 
         # Subscribe to a Topic.
         bounceEndpoint = reverse(getBounces)
@@ -565,16 +585,24 @@ def doSetNotificationTopic(email, useCache=False):
                 complaintTopic, 'http',
                 urljoin(settings.LOCAL_URL, complaintEndpoint))
 
+        deliveryEndpoint = reverse(getDeliveries)
+        if not hasDeliveryTopicExist:
+            sns.subscribe(
+                deliveryTopic, 'http',
+                urljoin(settings.LOCAL_URL, deliveryEndpoint))
+
         # Set notification topic.
         ses.setIdentityNotificationTopic('Bounce', email, bounceTopic)
         ses.setIdentityNotificationTopic('Complaint', email, complaintTopic)
+        ses.setIdentityNotificationTopic('Delivery', email, deliveryTopic)
         if useCache:
             # Set cache.
             setSenderByCache(email)
         return {
             'result': True,
             'bounceTopic': bounceTopic,
-            'complaintTopic': complaintTopic
+            'complaintTopic': complaintTopic,
+            'deliveryTopic': deliveryTopic
         }
     except Exception as e:
         return {
